import json
import re
import requests
from . import query
from bs4 import BeautifulSoup

from .utils import HEADERS


class PromotionsMixin:
    def get_promotions(self, business_id: str, is_booking:bool = False, proxies=None):
        data = query.get_promotions.create(business_id, is_booking)
        response = requests.post("https://pcmap-api.place.naver.com/graphql", headers=HEADERS, data=json.dumps(data), proxies=proxies)
        response.raise_for_status()
        response_data = response.json()
        graphql_data = response_data['data']
        graphql_data['business_id'] = business_id
        graphql_data['_id'] = business_id
        return graphql_data



def create(businessId: str, businessType: str = "restaurant", deviceType: str = "pcmap"):
    operation_name = "getAnnouncements"
    variables = _define_variables(businessId, businessType, deviceType)
    query = {
        "operationName": operation_name,
        "variables": variables,
        "query": QUERY_STATEMENT
    }
    return query


def _define_variables(businessId: str, businessType: str, deviceType: str):
    return {
        "businessId": businessId,
        "businessType": businessType,
        "deviceType": deviceType
    }


QUERY_STATEMENT = """
    query getAnnouncements($businessId: String!, $businessType: String!, $deviceType: String!) {
      announcements: announcementsViaCP0(
        businessId: $businessId
        businessType: $businessType
        deviceType: $deviceType
      ) {
        ...AnnouncementFields
        __typename
      }
    }

    fragment AnnouncementFields on Feed {
      feedId
      category
      title
      relativeCreated
      period
      thumbnail {
        url
        count
        isVideo
        __typename
      }
      __typename
    }
    """



def create(businessId: str, cid: str = "231053", type: str = 'PICKUP'):
    operation_name = "getPopularMenus"
    variables = _define_variables(businessId, cid, type)
    query = {
        "operationName": operation_name,
        "variables": variables,
        "query": QUERY_STATEMENT
    }
    return query


def _define_variables(businessId: str, cid: str = '231053', type='PICKUP'):
    return {
        "bookingBusinessId": businessId,
        "cid":cid,
        "itemType": type,
        "timeout": 3000
    }


QUERY_STATEMENT = """
     query getPopularMenus($input: PopularMenusInput) {
      popularMenus(input: $input) {
        hasPopular
        items {
          id
          name
          bookingCount
          price
          normalPrice
          imageUrl
          menuListUrl
          menuUrl
          isPopular
          images
          score
          __typename
        }
        __typename
      }
    }
"""



def create(businessId: str):
    operation_name = "restaurant"
    variables = _define_variables(businessId)
    query = {
        "operationName": operation_name,
        "variables": variables,
        "query": QUERY_STATEMENT
    }
    return query


def _define_variables(businessId: str):
    return {
        "deviceType": "pcmap",
        "businessId": businessId,
        "isNx": False
    }


QUERY_STATEMENT = """
query restaurant($businessId: String) {
  restaurant(id: $businessId, isNx: false, deviceType: "pcmap") {
    base {
      address
      baemin {
        businessHours {
          deliveryTime {
            start
            end
            __typename
          }
          closeDate {
            start
            end
            __typename
          }
          temporaryCloseDate {
            start
            end
            __typename
          }
          __typename
        }
        __typename
      }
      # bookingAwards
      bookingBusinessId
      bookingButtonName
      bookingDisplayName
      bookingHubButtonName
      broadcastInfos {
        program
        date
        menu
        __typename
      }
      businessHours {
        day
        description
        endTime
        hourString
        index
        isDayOff
        startTime
        __typename
      }
      category
      categoryCode
      categoryCodeList
      categoryCount
      cescoCheck
      # cescoLink
      cescofsCheck
      # cescofsLink
      chatBotUrl
      conveniences
      coordinate {
        x
        y
        mapZoomLevel
        __typename
      }
      dbType
      defaultCategoryCodeList
      description
      detailPath
      # driveThru
      # easyOrder
      # foodSafetyInfo
      giftCards
      gifticonId
      gifticonMoreUrl
      hasAiCall
      hasOfficialImages
      hasSmartOrder
      hideBusinessHours
      hidePrice
      homepages {
        # etc
        repr {
          isDeadUrl
          isRep
          landingUrl
          order
          type
          url
          __typename
        }
        __typename
      }
      id
      images {
        desc
        height
        infoTitle
        number
        origin
        source
        url
        width
      }
      imagesLastModified
      imagesLastModifiedDate
      is100YearCertified
      isEasyOrder
      isForcedTakeOut
      isKtis
      isRelaxRestaurant
      isStarbucks
      keywords
      mapUrl
      menuImages {
        imageUrl
      }
      # menuSource
      menus {
        change
        description
        id
        images
        index
        name
        price
        priceType
        priority
        recommend
      }
      # mfds
      # michelinGuide
      microReviews
      missingInfo {
        isBizHourMissing
        isMenuImageMissing
        isAccessorMissing
        isBoss
        isConveniencesMissing
        isDescriptionMissing
        needLargeSuggestionBanner
        __typename
      }
      moreBookingReviewsPath
      moreFsasReviewsPath
      morePhotosPath
      moreUGCReviewsPath
      nPayPromotion
      name
      # naverBlog
      naverBookingHubUrl
      # naverBookingPromotion
      naverBookingUrl
      newOpening
      # openingHours
      paymentInfo
      phone
      poiInfo {
        # polyline
        # polygon
        __typename
      }
      # preOrder
      promotionTitle
      rcode
      relatedLinks {
        clickCode
        iconName
        name
        url
        __typename
      }
      reportWrongInfoType
      reviewSettings {
        blog
        cafe
        keyword
        showVisitorReviewScore
        __typename
      }
      road
      roadAddress
      routeUrl
      siteId
      staticMapUrl
      streetPanorama {
        id
        lat
        lon
        pan
        tilt
        __typename
      }
      # tableOrder
      # takeOut
      talktalkUrl
      themes
      virtualPhone
      visitorReviewsScore
      visitorReviewsTextReviewTotal
      visitorReviewsTotal
      zoomLevel
      __typename
    }
    bookingItems {
      items {
        id
        businessId
        name
        isNPayUsed
        desc
        bookingUrl
        imageUrls
        bizItemType
        reviewStat {
          score
          count
          __typename
        }
        originalBookingUrl
        __typename
      }
      __typename
    }
    brandPromotion {
      brandDirNum
      brandLogoImage
      brandMobileUrl
      brandPcUrl
      brandPhone
      brandTitle
      eDate
      promotionDetail
      promotionEdate
      promotionEdayOfWeek
      promotionImage
      promotionMobileUrl
      promotionPcUrl
      promotionPlaceImage
      promotionSdate
      promotionSdayOfWeek
      promotionType
      sDate
      status
      __typename
    }
    busStations {
      displayCode
      id
      innerRoutes {
        routeType {
          innerRoute {
            id
            name
            __typename
          }
          type
          typeName
          __typename
        }
        __typename
      }
    }
    businessStats {
      age {
        ageKey
        rank
        value
        __typename
      }
      # alsoSearched
      # contexts
      daily {
        dayKey
        hourly {
          hourKey
          value
          __typename
        }
        __typename
      }
      gender {
        f
        m
        __typename
      }
      lastUpdated
      __typename
    }
    businessTools {
      tools {
        countInString
        description
        link
        title
        type
        using
        __typename
      }
      __typename
    }
    datalabPhotos {
      clusterId
      clusterName
      imageUrl
      items {
        orgPath
        __typename
      }
      __typename
    }
    excludeTargets {
      image
      imageSection
      review
      __typename
    }
    fsasReviews {
      items {
        authorName
        bySmartEditor3
        bypassToken
        contents
        createdString
        date
        hasNaverReservation
        home
        id
        isOfficial
        isRepresentative
        isVideoThumbnail
        name
        profileImageUrl
        rank
        reviewId
        thumbnailCount
        thumbnailUrl
        thumbnailUrlList
        title
        type
        typeName
        url
        __typename
      }
      maxItemCount
      total
      __typename
    }
    hasFeed {
      blogExist
      feedExist
      __typename
    }
    # kinQna
    # navercasts
    ncr {
      festivalSectionHeadInfo
      # festivals
      headInfo
      # performances
      __typename
    }
    newBusinessHours {
      businessHours {
        breakHours {
          start
          end
          __typename
        }
        businessHours {
          end
          start
          __typename
        }
        day
        description
        lastOrderTimes {
          time
          type
          __typename
        }
        __typename
      }
      businessStatusDescription {
        blindDescription
        description
        status
        __typename
      }
      # comingIrregularClosedDays
      comingRegularClosedDays
      freeText
      name
      __typename
    }
    shopWindow {
      # branchNews
      # itemsFromChannelNo
      # itemsFromInterlockId
      typeName
      url
      __typename
    }
    subwayStations {
      color
      name
      no
      priority
      station {
        id
        lat
        lng
        name
        nearestExit
        nearestExitType
        walkTime
        walkingDistance
        __typename
      }
      # transfers
      type
      typeDesc
      __typename
    }
    tabs {
      tabId
      __typename
    }
    # tvcasts
    visitorReviewStats {
      analysis {
        menus {
          count
          label
          __typename
        }
        themes {
          code
          count
          label
          __typename
        }
        votedKeyword {
          details {
            category
            code
            count
            displayName
            iconCode
            iconUrl
            previousRank
            __typename
          }

          reviewCount
          totalCount
          userCount
          __typename
        }
      }
      apolloCacheId
      id
      name
      ratingReviewsTotal
      review {
        authorCount
        avgRating
        imageReviewCount
        maxScoreWithMaxCount
        maxSingleReviewScoreCount
        scores {
          count
          score
          __typename
        }
        starDistribution {
          count
          score
          __typename
        }
        totalCount
        __typename
      }
      visitorReviewsTotal
      __typename
    }
  }

  sasImages(
    input: {
      businessId: $businessId
      businessType: "restaurant"
      display: 30
      excludeAuthorIds: []
      excludeSection: []
    }
  ) {
    items {
      authorThumbnail
      authorname
      date
      height
      imgUrl
      link
      originalUrl
      rank
      section
      subsection
      text
      title
      width
      __typename
    }
    relation
    total
    __typename
  }
  user {
    myplace {
      profile {
        borderImageUrl
        imageUrl
        __typename
      }
      __typename
    }
    partnerHashedIdNo
    __typename
  }
  visitorReviewMediasTotal (input: { businessId: $businessId, businessType: "restaurant"}) {
    total
  }
  visitorReviewStats (input: { businessId: $businessId, businessType: "restaurant"})  {
    analysis {
      menus {
        count
        label
        __typename
      }
      themes {
        code
        count
        label
        __typename
      }
      votedKeyword {
        details {
          category
          code
          count
          displayName
          iconCode
          iconUrl
          previousRank
          __typename
        }
        reviewCount
        totalCount
        userCount
        __typename
      }
    }
    # apolloCacheId
    id
    name
    ratingReviewsTotal
    review {
      authorCount
      avgRating
      imageReviewCount
      maxScoreWithMaxCount
      maxSingleReviewScoreCount
      scores {
        count
        score
        __typename
      }
      starDistribution {
        count
        score
        __typename
      }
      totalCount
      __typename
    }
    visitorReviewsTotal
    __typename
  }
  visitorReviews (input: { businessId: $businessId, businessType: "restaurant", 
    cidList: ["220036","220039","220091","220861"], 
    includeContent: true,
    page: 1,
    size:3
    })   {
    hideProductSelectBox
    items {
      author {
        borderImageUrl
        from
        id
        imageUrl
        isFollowing
        nickname
        objectId
        # review
        # theme 
        
        url
        __typename
      }
      body
      businessName
      created
      highlightOffsets
      id
      # item
      language
      media {
        thumbnail
        type
        videoId
        videoOriginSource
        __typename
      }
      originType
      rating
      receiptInfoUrl
      reply {
        body
        created
        editUrl
        editedBy
        isReported
        isSuspended
        replyTitle
        __typename
      }
      status
      tags
      thumbnail
      translatedText
      viewCount
      visitCount
      visited
      __typename
    }
    # starDistribution
    total
    __typename
  }
}

"""



import json
import math
import re
from seleniumwire import webdriver, utils
from selenium import webdriver
import time
import requests
from bs4 import BeautifulSoup
from seleniumwire import webdriver as wired_webdriver
from selenium.webdriver.common.by import By
from selenium.webdriver.chrome.service import Service
from webdriver_manager.chrome import ChromeDriverManager
from naverplaceapi.mixin.utils import HEADERS, parse_naver_var_in_script_texts
from . import query


def extract_graphql_request(driver):
    for request in driver.requests:
        request_url = request.url
        if "graphql" not in request_url:
            continue
        return request
    return None


def wait_for_graphql_response(driver,
                              max_attempts=10,
                              waiting_time=2
                              ):
    current_attempt = 0

    while current_attempt < max_attempts:
        need_to_waiting = False
        for request in driver.requests:
            request_url = request.url
            if "graphql" not in request_url:
                continue
            if request.response is None:
                need_to_waiting = True
                break
        if need_to_waiting == False:
            break
        time.sleep(waiting_time)


def _request_get_fsas_review_like_selenium(driver, place_id, page_no, page_cnt):
    # 수정된 요청 보내기
    import requests
    data = query.get_fsas_reviews.create(place_id, page_no, page_cnt)

    # 요청 새 세션 생성
    s = requests.Session()
    # Selenium Cookie 획득
    cookies = driver.get_cookies()
    for cookie in cookies:
        s.cookies.set(cookie['name'], cookie['value'])

    # 블로그 정보 요청 GraphQL의 헤더 정보를 획득
    request_headers = None
    graphql_request = extract_graphql_request(driver)
    if graphql_request is not None:
        request_headers = graphql_request.headers

    # 이전 요청과 동일한 형식으로 데이터 요청
    response = s.post("https://pcmap-api.place.naver.com/graphql", headers=request_headers, data=json.dumps(data))
    return response


def _request_get_visitor_review_like_selenium(driver, place_id, page_no, page_cnt):
    # 수정된 요청 보내기
    import requests
    data = query.get_visitor_reviews.create(place_id, page_no, page_cnt)

    # 요청 새 세션 생성
    s = requests.Session()
    # Selenium Cookie 획득
    cookies = driver.get_cookies()
    for cookie in cookies:
        s.cookies.set(cookie['name'], cookie['value'])

    # 블로그 정보 요청 GraphQL의 헤더 정보를 획득
    request_headers = None
    graphql_request = extract_graphql_request(driver)
    if graphql_request is not None:
        request_headers = graphql_request.headers

    # 이전 요청과 동일한 형식으로 데이터 요청
    response = s.post("https://pcmap-api.place.naver.com/graphql", headers=request_headers, data=json.dumps(data))
    return response


class ReviewMixin:
    def get_visitor_reviews(self, business_id: str, page_no: int, page_cnt: int, proxies=None):
        data = query.get_visitor_reviews.create(business_id, page_no, page_cnt)
        response = requests.post("https://pcmap-api.place.naver.com/graphql", headers=HEADERS, data=json.dumps(data),
                                 proxies=proxies)
        response.raise_for_status()
        response_data = response.json()
        graphql_data = response_data['data']['visitorReviews']
        if graphql_data is None:
            graphql_data = {}
        # ['visitorReviews']
        graphql_data['business_id'] = business_id
        return graphql_data

    def get_ugc_reviews(self, business_id: str, page_no: int, page_cnt: int, proxies=None):
        data = query.get_ugc_reviews.create(business_id, page_no, page_cnt)
        response = requests.post("https://pcmap-api.place.naver.com/graphql", headers=HEADERS, data=json.dumps(data),
                                 proxies=proxies)
        response.raise_for_status()
        response_data = response.json()
        graphql_data = response_data['data']['restaurant']['fsasReviews']
        graphql_data['business_id'] = business_id
        return graphql_data

    def get_visitor_review_stats(self, business_id: str, proxies=None):
        data = query.get_visitor_review_stats.create(business_id)
        response = requests.post("https://pcmap-api.place.naver.com/graphql", headers=HEADERS, data=json.dumps(data),
                                 proxies=proxies)
        response.raise_for_status()
        response_data = response.json()
        graphql_data = response_data['data']['visitorReviewStats']
        if graphql_data is None:
            return None
        graphql_data['_id'] = graphql_data['id']
        graphql_data['business_id'] = business_id
        return graphql_data

    def get_visitor_review_photos_in_visitor_review_tab(self, store_id: str, page_no: int, page_size: int,
                                                        proxies=None):
        data = query.get_visitor_review_photos_in_visitor_review_tab.create(store_id, page_no, page_size)
        response = requests.post("https://pcmap-api.place.naver.com/graphql", headers=HEADERS, data=json.dumps(data),
                                 proxies=proxies)
        response.raise_for_status()
        response_data = response.json()
        graphql_data = response_data['data']['visitorReviews']
        if graphql_data is None:
            graphql_data = {}
        graphql_data['business_id'] = store_id
        return graphql_data

    def get_visitor_review_theme_lists(self, store_id: str, page_no, page_size, proxies=None):
        data = query.get_visitor_review_theme_lists.create(store_id, page_no, page_size)
        response = requests.post("https://pcmap-api.place.naver.com/graphql", headers=HEADERS, data=json.dumps(data),
                                 proxies=proxies)
        response.raise_for_status()
        response_data = response.json()
        graphql_data = response_data['data']['themeLists']
        graphql_data['business_id'] = store_id

        return graphql_data

    def get_blog_reviews_in_html(self,
                                 business_id: str,
                                 page_no=1,
                                 page_size=10,
                                 use_tor:bool =False,
                                 tor_port:int =9150,
                                 proxies=None):
        page_size = max(min(page_size, 10), 1)

        options = webdriver.ChromeOptions()
        options.add_argument('--disable-dev-shm-usage')
        options.add_argument('headless')
        options.add_argument('--no-sandbox')

        if use_tor is True:
            tor_host = f'socks5://127.0.0.1:{tor_port}'
            options.add_argument(f'--proxy-server={tor_host}')

        driver = wired_webdriver.Chrome(service=Service(ChromeDriverManager().install()), options=options)

        def request_interceptor(request):
            if request.path.endswith(('.png', '.jpg', '.gif', 'jpeg')):
                request.abort()

        driver.request_interceptor = request_interceptor
        url = "https://pcmap.place.naver.com/restaurant/{}/review/ugc?type=photoView".format(business_id)
        driver.get(url)
        time.sleep(4)
        gql_response = _request_get_fsas_review_like_selenium(driver, business_id, page_no, page_size)

        gql_response.raise_for_status()
        response_data = gql_response.json()
        graphql_data = response_data['data']['fsasReviews']
        graphql_data['business_id'] = business_id
        return graphql_data

    def get_visitor_reviews_in_html(self,
                                    business_id: str,
                                    page_no=1,
                                    page_size=10,
                                    use_tor:bool = False,
                                    tor_port:int = 9150,
                                    proxies=None):
        page_size = max(min(page_size, 10), 1)

        options = webdriver.ChromeOptions()
        options.add_argument('--disable-dev-shm-usage')
        options.add_argument('headless')
        options.add_argument('--no-sandbox')

        if use_tor is True:
            tor_host = f'socks5://127.0.0.1:{tor_port}'
            options.add_argument(f'--proxy-server={tor_host}')

        driver = wired_webdriver.Chrome(service=Service(ChromeDriverManager().install()), options=options)

        def request_interceptor(request):
            if request.path.endswith(('.png', '.jpg', '.gif', 'jpeg')):
                request.abort()

        driver.request_interceptor = request_interceptor

        url = "https://pcmap.place.naver.com/restaurant/{}/review/ugc?type=photoView".format(business_id)
        driver.get(url)
        time.sleep(4)
        gql_response = _request_get_visitor_review_like_selenium(driver, business_id, page_no, page_size)

        gql_response.raise_for_status()
        response_data = gql_response.json()
        graphql_data = response_data['data']['fsasReviews']
        graphql_data['business_id'] = business_id
        return graphql_data
